/*
 1bit serial driver for PIC
  copyright(C) T.Satoh  mail:T-Satoh@NetLaputa.ne.jp
   Ver 0.01 made by T.Satoh 2005/11/19
   Ver 0.02 improved to be shorter codes by T.Satoh 2005/11/23
   Ver 0.03 modified for boot loader by T.Satoh 2006/01/03
   Ver 0.04 modify error chech and shorter code by T.S 2006/01/22
   Ver 0.05 change a little by T.S 2008/11/23
   Ver 0.06 be able to set polality sepalately

phisical format(start:1 data:8 stop:1)
  1____      ____ ____ ____ ____ ____ ____ ____ ____ ____
       X____X____X____X____X____X____X____X____X____X
  0     Start LSB bit1 bit2 bit3 bit4 bit5 bit6  MSB Stop

usage
  see serial.h
*/


#if SERIAL_SND_POLARITY != 0
  #define SND_SIO_ON 1
  #define SND_SIO_OFF 0
#else
  #define SND_SIO_ON 0
  #define SND_SIO_OFF 1
#endif

#if SERIAL_RCV_POLARITY != 0
  #define RCV_SIO_ON 1
  #define RCV_SIO_OFF 0
#else
  #define RCV_SIO_ON 0
  #define RCV_SIO_OFF 1
#endif

/* wait n*3+1 clocks */
#define SIO_BIT_WAIT( label, n )	\
	SioWaitCounter = n;				\
label:								\
	SioWaitCounter = decsz( SioWaitCounter ); \
	goto label

//#pragma library 1

//shrBank char SioData;		// receive/send data
//shrBank char SioBitCnt;		// bit counter
//shrBank char SioWaitCounter;	// wait timer counter
//shrBank uns16 SioLoopCnt;

#pragma updateBank 0

/*
  send 1byte
   SioData <- send data
*/
void SioPutchar( uns8 W )
{
//	STATUS = 0;	// clear RP0,RP1
	SioData = W;

	SioBitCnt = 10;			// set output bit count(1+8+1)

#ifdef RS232C_SET_OUT
	RP0 = 1;
	RP1 = 0;
	RS232C_SET_OUT;
	RP0 = 0;
#endif
#if SERIAL_SND_POLARITY == 0
	SioData = SioData ^ 255;		// COMF  SioData,1
#endif
	Carry = SND_SIO_OFF;		// set start bit SioData
sioput:
		RS232_OUTBIT = Carry;	// 4(4)
		SIO_BIT_WAIT( sio_p00, SIO_BIT_TIM ); // 43=14*3+1(47)
		Carry = SND_SIO_ON;			// 1(48)
		SioData = rr( SioData );		// 1(49)
		SioBitCnt = decsz( SioBitCnt ); // 1(50)
		goto sioput;			// 2,1(52,51)
#ifdef RS232C_SET_INP
	RP0 = 1;
//	RP1 = 0;
	RS232C_SET_INP;
	RP0 = 0;
#endif
}


/*
 receive data
  return -> receive datavalue
  RP1 -> 1:data exist, 0:no data
*/
uns8 SioGetchar( void )
{
//	STATUS = 0;	// clear RP0,RP1

	if( ErrFlg == 0 ) {	// if error occured
		return( W );
	} else {
		RP1 = 0;
#if 1
		SioLoopCnt.high8 = (64 * (1<<(SERIAL_SPEED-1))) & 0xff;
#else
		SioLoopCnt.high8 = 0xff;
#endif
		do {
			do {		// 5clk loop * 256 * 2 = 2560uS(4MHz)
				if( RS232_INPBIT == RCV_SIO_OFF ) {	// 2(2) if start bit
					goto siog00;
				}
			} while( --SioLoopCnt.low8 );	// 3(5)
		} while( --SioLoopCnt.high8 );
siog00:	if( SioLoopCnt.high8 ) {			// 3(8) not time out
			SioWaitCounter = SIO_BIT_TIM + 5;	// 2(10,max:+4)
			SioBitCnt = 9;					// 2(12)
			goto siog01;					// 2(14)
sioget:
				SioWaitCounter = SIO_BIT_TIM;	// 2(2)
siog01:			SioWaitCounter = decsz( SioWaitCounter ); // 41=14*3-1(43) (70=19*3-1+14)
				goto siog01;
				Carry = RS232_INPBIT;			// 3(46)(73)
				SioData = rr( SioData );	// 1(47)(74,max:+4:purpose=78=52*1.5)
				nop2();							// 2(49)
				SioBitCnt = decsz( SioBitCnt );				// 1(50)
				goto sioget;					// 2(52)
			SioData = rl( SioData );		// if Carry=SIO_ON then stop bit ok
			if( Carry == RCV_SIO_ON ) {		// if stop bit exist
				ErrFlg = 1;	// set error flag to OK
			}
#if SERIAL_RCV_POLARITY == 0
			SioData = SioData ^ 255;	// COMF  SioData,1
#endif
		}
//		W = SioData;
		ChkSum += SioData;
	}
	return( W );
}

#pragma updateBank 1


//#pragma library 0
