/*******************************************
 OneBitLoader for PIC24FJ64
  copyright(c) skyriver Ver 0.08 2015/06/28
   correct Body part serial data(add address)
  copyright(c) skyriver Ver 0.01 2015/06/16
 *******************************************/

	.title	"OneBitLoader for PIC24FJ"

	.include "OneBitLoader.inc"

.IF (COMPACT_HEX != 0)
	.section	.text_DummyAp,code,address( DummyApAdrs );
.ELSE
	.text
.ENDIF

	.global __reset

; flash prom write key sequence
; W5 <- NVMCON command
.macro	KEY_SEQUENCE
	mov		W5,NVMCON
	mov		#0x55,W0
	mov		W0,NVMKEY
	mov		#0xaa,W0
	mov		W0,NVMKEY
	bset	NVMCON,#WR
	nop
	nop
0:	 btsc	NVMCON,#WR
	 bra	0b
	return
.endm



; +++++ preinstalled dummy application ++++++
;  echo back serial input character

DummyAp:
	bset	CORCONL,#PSV
	mov		#psvpage(Welcome),W0
	mov		W0,PSVPAG
	mov		#psvoffset(Welcome),W1
	rcall	Puts
0:
	rcall	DmGetchar
	bra		NC,0b
	mov		W0,W2			; save input character
	mov		#psvoffset(Answer),W1
	rcall	Puts
	mov		W2,W0
	call	iPutchar
	mov		#psvoffset(CrLf),W1
	rcall	Puts
	bra		0b


Puts:
	clr		W0
0:
	mov.b	[W1++],W0
	cp0.b	W0
	btsc	SR,#Z
	return
	call	iPutchar
	bra		0b


DmGetchar:
	mov	#( StepRate / (1000 / RcvTim ) / 25 ),W0
0:
	BTS_STARTBIT		; 1(1)
	bra		1f			; 1(2)
	repeat	#(19 - 1)	; 1(3)
	nop					; 19(22)
	dec		W0,W0		; 1(23)
	bra		C,0b		; 2(25)
	return				; time over(carry:0)
1:
	repeat	#(BitClk * 3 / 2 - 5)	; 1(4)
	mov		#0x0080,W0	; clear receive data and set loop mark
2:
	 BTS_BIT0			; 1(1)
	 bset	W0,#8		; 1(2)
	 repeat	#(BitClk - 7) ; 1(3)
	 nop				; BitClk - 6
	 lsr	W0,W0		; 1(4)
	 bra	NC,2b		; 2(6)
	BTS_BIT1			; check stop bit
	bclr	SR,#C		; if stop bit error(carry:0)
	return


Welcome:	; must even length to connect next CrLf
	.ascii	"\rWelcome to OneBitLoader V0.08 by skyriver 2015/06/16\r\n  Please push any key"
CrLf:
	.asciz	"\r\n"
Answer:
	.asciz	"You pushed : "



; +++++ OneBitLoader(body part) ++++++

.IF (COMPACT_HEX != 0)
	.section	.text_body,code,address( BodyAdrs )
.ELSE
	.org	BodyAdrs - ADRS_OFFSET
.ENDIF


;  W0 work
;  W1 word counter
;  W2 data counter/EA
;  W3 buffer pointer
;  W4 work
;  W5 NVMCON command(receive CheckSum)

BootLoader:
BI_00:
	 mov	#0xffff,W3		; set 0x00ffffff to rest of row word
	 and	W2,W4,W0		; check row end
	 bra	Z,BI_20			; if just end

BI_10:
	  tblwth	W3,[W2]
	  tblwtl	W3,[W2++]
	  and	W2,W4,W0		; check row end
	  bra	NZ,BI_10
	 rcall	BWriteKey		; write rest datas

BI_20:
	  tblwtl	W3,[--W2]	; clear latch with 0xffff for next write
	  tblwth	W3,[W2]
	  and		W2,W4,W0	; check row end
	  bra		NZ,BI_20

	 rcall	iAckSend		; send ACK
	 clr	W5				; clear CheckSum
	 rcall	iGetchar
	 mov.b	WREG,WREG1L		; save data word counter(low byte)
	 rcall	iGetchar
	 mov.b	WREG,WREG1H		; save data word counter(high byte)
	 mul.uu	W1,#3,W2		; calc receive data
	 inc2	W2,W2			; add command and CheckSum
	 mov	#RcvBuf,W3
B_10:
	  rcall	iGetchar
	  mov.b	W0,[W3++]
	  dec	W2,W2
	  bra	NZ,B_10

	 cp0.b	W5				; check CheckSum
	 bra	NZ,ApJmp

	 mov	#RcvBuf,W3
	 mov.b	[W3++],W4		; get command character
	 cp.b	W4,#MAX_CMD
	 bra	C,ApJmp			; if illigal command
	 dec	W1,W1			; decriment word counter(exact address)
	 mov	[W3++],W2		; get EA(AdrL and AdrH) W3 is even address
	 mov.b	[W3++],W0		; get AdrU
	 cp0.b	W4

; IPL data write proc(last data)
	 bra	Z,ReEntry			; W1:word counter, W3:buffer address

B_20:
	 mov	W0,TBLPAG
	 dec.b	W4,W4			; decriment commnad No
	 bra	NZ,B_30			; if not erase

	 tblwtl.b	[W3],[W2]	; set erase page
	 mov	#( 1<<WREN | 1<<ERASE | 1<<NVMOP1),W5	; page erase 0x4042
	 rcall	BWriteKey
B_30:
	 mov	#( 2 * ROW_SIZE - 1),W4	; set row end check mask
	 mov	#( 1<<WREN | 1<<NVMOP0),W5	; set page write command
B_40:
	  tblwth.b	[W3++],[W2]		; high(low byte of upper word)
	  tblwtl.b	[W3++],[W2++]	; low(low byte of lower word)
	  tblwtl.b	[W3++],[W2++]	; middle(high byte of lower word)
	  and	W2,W4,W0			; check row end
	  btsc	SR,#Z
	  rcall	BWriteKey
	  dec	W1,W1
	  bra	NZ,B_40
	 bra	BI_00


; write key sequence
;  W5 <- NVMCON command

BWriteKey:
	KEY_SEQUENCE


; +++++ IPL of OneBitLoader ++++++

.IF (COMPACT_HEX != 0)
	.section	.text_Ipl,code,address( IplAdrs )
.ELSE
	.org	IplAdrs - ADRS_OFFSET
.ENDIF


__reset:


; IPL : initial program loader(only IPL is in program memory)
;  W0 work
;  W1 word counter
;  W2 data counter
;  W3 buffer pointer
;  W4
;  W5 NVMCON command(receive CheckSum)

Ipl:
	mov		#PIC_ID,W0
	rcall	Putchar			; send pic id
	rcall	Getchar
	mov		W0,W1			; save word counter
	mul.uu	W0,#3,W2		; calc data suu(W3 destroyed)
	mov		#RcvBuf,W3		; set buffer pointer
Ipl_000:
	 rcall	Getchar
	 mov.b	W0,[W3++]
	 dec	W2,W2
	 bra	C,Ipl_000		; loop W2 + 1(CheckSum)

	cp0.b	W5				; check CheckSum
	bra		NZ,ApJmp

	mov		#tblpage(BootLoader),W0
	mov		#tbloffset(BootLoader),W2
	mov		#RcvBuf,W3
	rcall	PromWrite
	bra		BootLoader


; erase and write prom
;  W0 <- TBLPAG value
;  W1 <- word count
;  W2 <- EA
;  W3 <- buffer address

PromWrite:
	mov		W0,TBLPAG
	tblwtl.b	[W3],[W2]			; set erase page
	mov		#( 1<<WREN | 1<<ERASE | 1<<NVMOP1),W5	; page erase 0x4042
	rcall	WriteKey			; erase PAGE
	mov		#(2 * ROW_SIZE - 1),W4	; set row end check mask
	mov		#(1<<WREN | 1<<NVMOP0),W5	; set page write command
0:
	 tblwth.b	[W3++],[W2]		; high(low byte of upper word)
	 tblwtl.b	[W3++],[W2++]	; low(low byte of lower word)
	 tblwtl.b	[W3++],[W2++]	; middle(high byte of lower word)
	 and	W2,W4,W0			; check row end
	 btsc	SR,#Z
	 rcall	WriteKey
	 dec	W1,W1
	 bra	NZ,0b
	return



; write key sequence
;  W5 <- NVMCON command

WriteKey:
	KEY_SEQUENCE


; send serial(8bit,non-pari,stop:1)
;  W0 <- send data

Putchar:
	add		#0x0300,W0	; set loop mark, stop bit & start bit(carry:0)
0:
	 btss	SR,#C		; 1(1)
	 SERI_OUT_0			; 1(2)
	 btsc	SR,#C		; 1(3)
	 SERI_OUT_1			; 1(4)
	 repeat	#(BitClk - 9)	; 1(5)
	 nop				; BitClk - 8(adjust loop time to 1bit time)
	 lsr	W0,W0		; 1(6)
	 bra	NZ,0b		; 2(8)
	return


; receive serial(8bit,non-pari,stop:1)
;  W0 -> recieve data(if Carry:1)
;  Carry -> 0:timeover 1:receive data
;  Zero  -> 0:vertical CheckCum ok

Getchar:
	mov	#( StepRate / (1000 / RcvTim ) / 25 ),W0

; receive serial(8bit,non-pari,stop:1)
;  W0 <- timeover parameter( x 25 step )

GetcTim:
	BTS_STARTBIT		; 1(1)
	bra		Getc100		; 1(2)
	repeat	#(19 - 1)	; 1(3)
	nop					; 19(22)
	dec		W0,W0		; 1(23)
	bra		C,GetcTim	; 2(25)
	bra		ApJmp
Getc100:
	repeat	#(BitClk * 3 / 2 - 5)	; 1(4)
	mov		#0x0080,W0	; clear receive data and set loop mark
Getc200:
	 BTS_BIT0			; 1(1)
	 bset	W0,#8		; 1(2)
	 repeat	#(BitClk - 7) ; 1(3)
	 nop				; BitClk - 6
	 lsr	W0,W0		; 1(4)
	 bra	NC,Getc200	; 2(6)
	add.b	W0,W5,W5	; calc CheckSum
	BTS_BIT1			; check stop bit
	bra		ApJmp		; if stop bit error
	return


;
; don't change below address
;
iAckSend:
	mov		#ACK,W0
iPutchar:
	bra		Putchar
iGetchar:
	bra		Getchar


; reentry point from bootloader(body)
;  W0 <- TBLPAG value
;  W1 <- word count
;  W2 <- EA
;  W3 <- buffer address

ReEntry:
	rcall	PromWrite
	rcall	iAckSend
ApJmp:
	goto	DummyAp		; 2 words

IplEnd:

;	config2
;	config1

	.end
